#pragma once

#include <cstdint>
#include <sstream>
#include "JSON/json_reader.h"
#include "Systems/Config/RemoteConfigManager.h"

namespace Jetpack
{
	template<typename T>
	inline Configuration<T>::Configuration(const char* nameSpace, const char* key, int version)
	{
		std::stringstream stream;
		stream << nameSpace << "_" << key << "_" << version;
		m_configKey = stream.str();
	}

	template<typename T>
	inline Configuration<T>::~Configuration()
	{}

	template<typename T>
	inline bool Configuration<T>::Sync()
	{
		static_assert(sizeof(T) == 0, "Configuration with data type is not supported!");
		return false;
	}

	template<>
	inline bool Configuration<double>::Sync()
	{
		double value;
        bool isAvailable = Singleton<RemoteConfigManager>::Get().GetDoubleValue(
			GetConfigKey(),
			value);

		if (isAvailable == false)
		{
			return false;
		}

		Deserialise(value);
		return true;
	}

	template<>
	inline bool Configuration<int64_t>::Sync()
	{
		int64_t value;
		bool isAvailable = Singleton<RemoteConfigManager>::Get().GetIntegerValue(
			GetConfigKey(),
			value);

		if (isAvailable == false)
		{
			return false;
		}

		Deserialise(value);
		return true;
	}

	template<>
	inline bool Configuration<bool>::Sync()
	{
		bool value;
		bool isAvailable = Singleton<RemoteConfigManager>::Get().GetBooleanValue(
			GetConfigKey(),
			value);

		if (isAvailable == false)
		{
			return false;
		}

		Deserialise(value);
		return true;
	}

	template<>
	inline bool Configuration<std::string>::Sync()
	{
		std::string value;
		bool isAvailable = Singleton<RemoteConfigManager>::Get().GetStringValue(
			GetConfigKey(),
			value);

		if (isAvailable == false)
		{
			return false;
		}

		Deserialise(value);
		return true;
	}

	template<>
	inline bool Configuration<Json::Value>::Sync()
	{
		std::string value;
		bool isAvailable = Singleton<RemoteConfigManager>::Get().GetStringValue(
			GetConfigKey(),
			value);

		if (isAvailable == false)
		{
			return false;
		}
        
        Json::Value jsonValue;
        Json::Reader reader;
        bool result = reader.parse(value, jsonValue);
        
        if (result == false)
        {
            LOG_E("Failed to parse jason data from remote config for key %s", GetConfigKey());
            return false;
        }

		Deserialise(jsonValue);
		return true;
	}

	template<typename T>
	inline const char* Configuration<T>::GetConfigKey() const
	{
		return m_configKey.c_str();
	}
}
