#pragma once

#include <cstdint>
#include <sstream>
#include "util/JSONFile/JSONFile.h"

namespace Jetpack
{
    template<typename T>
    inline ConfigurationFileBased<T>::ConfigurationFileBased(const char* nameSpace, const char* key, int version) :
        Configuration<T>(nameSpace, key, version)
    {
        if (nameSpace == std::string("script"))
        {
            PANIC("script namespace is reserved and can't be used as namespace!");
            return;
        }
        
        std::stringstream stream;
        stream << "Configs/" << nameSpace << "/" << key << "_" << version << ".json";
        m_configPath = stream.str();
    }
    
    template<typename T>
    inline bool ConfigurationFileBased<T>::Sync()
    {
        if (Configuration<T>::Sync() == true)
        {
            return true;
        }
        
        // try to load from a local file if nothing loaded from remote config
        Util::JSONFile jsonFile(m_configPath.c_str(), Util::JSONFile::DiscardChangesOnDestroy);
        const bool didLoad = jsonFile.Load();
        if (didLoad == false)
        {
            return false;
        }
        
        return DeserialiseFromFile(jsonFile.Value());
    }
    
    template<typename T>
    inline bool ConfigurationFileBased<T>::DeserialiseFromFile(Json::Value& fileData)
    {
        static_assert(sizeof(T) == 0, "Configuration with data type is not supported!");
        return false;
    }
    
    template<>
    inline bool ConfigurationFileBased<double>::DeserialiseFromFile(Json::Value& fileData)
    {
        if (fileData.isDouble() == false)
        {
            return false;
        }
        
        return Deserialise(fileData.asDouble());
    }
    
    template<>
    inline bool ConfigurationFileBased<int64_t>::DeserialiseFromFile(Json::Value& fileData)
    {
        if (fileData.isInt64() == false)
        {
            return false;
        }
        
        return Deserialise(fileData.asInt64());
    }
    
    template<>
    inline bool ConfigurationFileBased<bool>::DeserialiseFromFile(Json::Value& fileData)
    {
        if (fileData.isBool() == false)
        {
            return false;
        }
        
        return Deserialise(fileData.asBool());
    }
    
    template<>
    inline bool ConfigurationFileBased<std::string>::DeserialiseFromFile(Json::Value& fileData)
    {
        if (fileData.isString() == false)
        {
            return false;
        }
        
        return Deserialise(fileData.asString());
    }
    
    template<>
    inline bool ConfigurationFileBased<Json::Value>::DeserialiseFromFile(Json::Value& fileData)
    {
        return Deserialise(fileData);
    }
}
