#pragma once

#include "Scripting/LuaIncludes.h"
#include "LuaSystem/LuaScript.h"


namespace Jetpack
{
	template<typename T> inline bool ConfigurationScript::Run(T& result)
	{
	    if (m_script.empty() == true)
	    {
	        LOG_E("ConfigurationScript:: Script is empty: {0}",  m_scriptPath.c_str());
	        return false;
	    }
	    
	    if (Lua::IsLuaActive() == false)
	    {
	        LOG_E("ConfigurationScript:: Failed to run {0}! Lua is not available.",  m_scriptPath.c_str());
	        return false;
	    }
	    
	    if (Lua::GetVM()->RunString(m_script.c_str(), 1) == false)
	    {
	        LOG_E("ConfigurationScript:: Failed to run {0}!",  m_scriptPath.c_str());
	        return false;
	    }

		// get returned value
		if (Lua::GetVM()->Get<T>(-1, result) == false)
		{
			LOG_E("ConfigurationScript:: Failed to get return value from {0}!",  m_scriptPath.c_str());
	        return false;
		}

		// pop returned value
		Lua::GetVM()->Pop(1);

	    return true;
	}

	inline bool ConfigurationScript::RunMultiReturn(int& returnCount)
	{
		if (m_script.empty() == true)
		{
			LOG_E("ConfigurationScript:: Script is empty: {0}", m_scriptPath.c_str());
			return false;
		}

		if (Lua::IsLuaActive() == false)
		{
			LOG_E("ConfigurationScript:: Failed to run {0}! Lua is not available.", m_scriptPath.c_str());
			return false;
		}

		if (Lua::GetVM()->RunStringMultiReturn(m_script.c_str(), returnCount) == false)
		{
			LOG_E("ConfigurationScript:: Failed to run {0}!", m_scriptPath.c_str());
			return false;
		}

		return true;
	}

	template<typename T> bool ConfigurationScript::GetNextReturnValue(T& value)
	{
		value = Lua::GetReturnValue<T>();
		return true;
	}
}
