#pragma once

#include <stdexcept>

namespace Jetpack
{

template<std::size_t size>
constexpr ConstExprString::ConstExprString(const char(&str)[size]) :
	m_value(str),
	m_size(size - 1)
{}

inline constexpr const std::size_t ConstExprString::GetSize() const
{
	return m_size;
}

template<std::size_t size>
inline constexpr std::size_t ConstExprString::GetSize(const char(&)[size])
{
	return size;
}

inline constexpr std::size_t ConstExprString::GetSizeUnsafe(char const* str)
{
	return *str ? 1 + GetSizeUnsafe(str + 1) : 0;
}

inline constexpr const char* ConstExprString::GetValue() const
{
	return m_value;
}

inline constexpr char ConstExprString::GetCharAt(std::size_t index) const
{
	return index < m_size ? m_value[index] : 0;
}

inline constexpr char ConstExprString::operator[](size_t index) const
{
	return GetCharAt(index);
}

inline constexpr bool compare_strings(const char* s1, const char* s2)
{
	return	(!*s1 && !*s2) ? // reached end of both?
				true : // all characters checked and they were equal
					(!*s1 || !*s2) ? // if reached end of either one
						false : // they are not equal
						*s1 == *s2 ? // otherwise compare the current characters
							compare_strings(s1+1, s2+1) : // proceed to check next chars
							false; // they are not equal
}

inline constexpr bool ConstExprString::operator==(const char* string) const
{
	return compare_strings(m_value, string);
}

inline constexpr std::size_t ConstExprString::Find(const ConstExprString& searchStr) const
{
	return Find(m_value, m_size, searchStr.GetValue(), searchStr.GetSize());
}

inline constexpr std::size_t ConstExprString::Find(const char* searchStr, size_t searchLength) const
{
	return Find(m_value, m_size, searchStr, searchLength);
}

inline constexpr std::size_t ConstExprString::Find(const char* str, size_t length,
	const char* searchStr, size_t searchLength)
{
	return searchLength == 0 ?	// have we reached end of the search string?
			length :	// Yes, return current position of the string being search
			length == 0 ?	// No, check if we have reached end of the string being searched
				(size_t)-1 :	// Yes, the search string couldn't be found
				str[length - 1] == searchStr[searchLength - 1] ?	// No, compare current character of string
																	// aginst current character of search string, are they equal?
					Find(str, length - 1, searchStr, searchLength - 1) :	// Yes, continue searching next characters
					Find(str, length - 1, searchStr, searchLength);	// No, reset search string current position
																	// and proceed to next character of the string being searched
}

template<typename StringContainer, size_t start,
	size_t length, char... suffix>
struct Substring
{
	static constexpr ConstExprString value()
	{
		return Substring<StringContainer,
			start, length - 1,
			StringContainer::value[start + length - 1],
			suffix...>::value();
	}
};

// teminator when length reaches zero
template<typename StringContainer, size_t start, char... suffix>
struct Substring<StringContainer, start, 0, suffix...>
{
	static constexpr ConstExprString value()
	{
		return ConstExprString(data);
	}

private:
	static constexpr const char data[] = { suffix..., 0 };
};

template<typename StringContainer, size_t start, char... suffix>
constexpr const char Substring<StringContainer, start, 0, suffix...>::data[];

template<typename StringContainer, size_t start, size_t length>
inline constexpr ConstExprString ConstExprString::GetSubString()
{
	return Substring<StringContainer, start, length>::value();
}

}
