#pragma once

#include "Framework/Util/ConstExprCrc.h"

#ifdef _MSC_VER
#define __USER_PROGRESS_TYPE_NAME__ __FUNCSIG__
#elif defined(__clang__) || defined(__GNUG__)
#define __USER_PROGRESS_TYPE_NAME__ __PRETTY_FUNCTION__
#else
static_assert(false, "Function name macro is not supported on current compiler!")
#endif

namespace Jetpack
{
	template<typename T> inline uint64_t GetUserProgressTypeId()
	{
		constexpr ConstExprString typeName(__USER_PROGRESS_TYPE_NAME__ );
		constexpr uint64_t typeId = GenerateCrc64(typeName);
		return typeId;
	}

	template<typename T> inline void IUserProgress::Register()
	{
		auto progress = std::shared_ptr<T>(new T());

		uint64_t typeId = GetUserProgressTypeId<T>();

		if (m_registeredProgresses.find(typeId) != m_registeredProgresses.end())
		{
			PANIC("IUserProgress:: %s has already been registered!", progress->GetId());
			return;
		}

		if (progress->IsPersistentAcrossInstalls())
		{
			progress->DeserialiseCloud();
		}
		else
		{
			progress->DeserialiseLocal();
		}

		m_registeredProgresses[typeId] = progress;
	}

	template<typename T> inline T* IUserProgress::Get()
	{
		uint64_t typeId = GetUserProgressTypeId<T>();

		auto itor = m_registeredProgresses.find(typeId);
		if (itor == m_registeredProgresses.end())
		{
			return nullptr;
		}

		return static_cast<T*>(itor->second.get());
	}
}
