﻿#pragma once

namespace Jetpack
{
	template<typename StateId>
	StateManager<StateId>::~StateManager()
	{
		for (auto it = m_states.begin(), end = m_states.end(); it != end; ++it)
		{
			delete it->second;
		}
		m_states.clear();
	}

	template<typename StateId>
	void StateManager<StateId>::AddListener(IStateListener<StateId>* listener)
	{
		m_listeners.push_back(listener);
	}

	template<typename StateId>
	void StateManager<StateId>::RemoveListener(IStateListener<StateId>* listener)
	{
		for (auto it = m_listeners.begin(), end = m_listeners.end(); it != end; ++it)
		{
			// I know this is expensive but it keeps the listener order the same, which may be important
			if ((*it) == listener)
			{
				m_listeners.erase(it);
				break;
			}
		}
	}

	template<typename StateId>
	void StateManager<StateId>::AddState(StateId stateId, State<StateId>* state)
	{
		m_states[stateId] = state;
	}

	template<typename StateId>
	State<StateId>* StateManager<StateId>::GetState(StateId stateId) const
	{
		auto it = m_states.find(stateId);
		if (it == m_states.end())
		{
			return nullptr;
		}

		return it->second;
	}

	template<typename StateId>
	void StateManager<StateId>::Update()
	{
		if (m_currentState == nullptr)
		{
			return;
		}

		m_currentState->Update();

		for (IStateListener<StateId>* listener : m_listeners)
		{
			listener->OnStateUpdate(m_currentState);
		}
	}

	template<typename StateId>
	void StateManager<StateId>::ChangeState(StateId newState)
	{
		if (m_currentStateId == newState && m_currentState != nullptr)
		{
			return;
		}

		if (m_currentState != nullptr)
		{
			m_currentState->Exit();

			for (IStateListener<StateId>* listener : m_listeners)
			{
				listener->OnStateExit(m_currentState);
			}
		}

		m_currentStateId = newState;
		m_currentState = GetState(m_currentStateId);

		if (m_currentState == nullptr)
		{
			PANIC("State not found");
			return;
		}

		m_currentState->Enter();

		for (IStateListener<StateId>* listener : m_listeners)
		{
			listener->OnStateEnter(m_currentState);
		}
	}
}
